

# Simplified Plurality Loss Probability Calculator #
# Replication file
# Sept 2021 #

# R version 4.1.1

setwd("your_working_directory_path_here")

library(foreign)
library(tidyverse)



######## Plurality Loss Probabilities (PLPR) #########

d <- read.csv("Elections_ParlGov.csv") #Organized Country-election-party

pg <- d %>% # pg for "parlgov"
  filter(election_type=="parliament") %>%  #remove EP elections
  filter(as.Date(election_date) > as.Date("1990-01-01")) %>% # remove old elecs
  group_by(country_name_short, election_date) %>%
  arrange(party_name, .by_group=TRUE) %>% 
  mutate(seats = ifelse(party_name=="Christlich Demokratische Union", seats+lead(seats), seats)) %>% #add CSU seats to CDU seats
  mutate(party_id = ifelse(party_id==808,1727,party_id)) %>% #replace CDU party_id w that of CDU/CSU
  mutate(seat_share = (seats / seats_total)*100) %>%  
  # when seatshare tie for first place in EU country, ensure that party that becomes PM is epsilon higher so that sort works below
  mutate(seat_share = if_else(country_name_short == "BEL" & party_name_short =="PS" & election_date=="2003-05-18", 16.666, seat_share)) %>% # prevents Belgian tie for 1st place
  mutate(seat_share = if_else(country_name_short == "ISL" & party_name_short =="Sj" & election_date=="2013-04-27", 30.158, seat_share)) %>% # prevents Icelandic tie for 1st place
  mutate(seat_share = if_else(country_name_short == "EST" & party_name_short =="EK" & election_date=="2003-03-02", 27.722, seat_share)) %>% # prevents Estonian tie for 1st place
  mutate(seat_share = if_else(country_name_short == "CYP" & party_name_short =="DISY" & election_date=="2006-05-21", 32.142, seat_share)) %>% # prevents Cypriot tie for 1st place
  arrange(country_name_short, election_date, desc(seat_share), .by_group=TRUE) %>% 
  mutate(plurparty = if_else(seat_share == sort(seat_share[1]),1,0)) %>% #bug: ties!
  mutate(runnerup = if_else(seat_share == sort(seat_share[2]),1,0)) %>% #bug: ties!
  mutate(tie = if_else(seat_share==lag(seat_share),1,0)) %>% #dummy for second obs in tie
  replace_na(list(tie = 0)) %>% #lag above will cause plurality party to have NA value for tie var
  mutate(seat_gap = seat_share - lead(seat_share)) %>% #lead (seat_share) is runner-up party
  ungroup() %>%
  group_by(country_id, party_id) %>%
  arrange(election_date, .by_group=TRUE) %>%
  mutate(sswing = seat_share - lag(seat_share)) %>% #use if new party prev seatshare should be NA; for 0 use following line
  group_by(country_id) %>% 
  arrange(election_date, runnerup, .by_group=TRUE) %>% 
  filter(plurparty ==1 | runnerup==1) %>% 
  arrange(country_id, election_date, seat_share, .by_group=TRUE) %>% #ascending
  mutate(second2plur = case_when(plurparty==1 & lag(party_id)==lead(party_id,2)~1,
                                 plurparty==1 & lag(party_id)!=lead(party_id,2)~0,
                                 plurparty==0 ~ 0, TRUE~NA_real_)) %>% #sorted so that lag(party_id) of plurparty is runner-up; lead(lead(party_id)) is plurparty in next elec
  ungroup() %>% 
  group_by(country_id, election_date) %>% # was line 54
  arrange(country_id, election_date, seat_share, .by_group=TRUE) %>% #ascending
  #define netswing as (B2-B1)-(A2-A1).  
  mutate(netswing = lag(sswing)-sswing)  %>% # runner-up's sswing - plur party's sswing (B-A)
  filter(plurparty==1) %>% 
  ungroup()%>%
  group_by(country_id)%>%
  arrange(country_id,election_date)%>%
  mutate(plurlost = if_else(party_id==lead(party_id),0,1))%>% # dummy for plur party being replaced in next elec (39% of time); for validity check
  ungroup() %>% 
  group_by(country_id) %>%
  mutate(n_netswing=sum(!is.na(netswing))) %>% #alternative to add_count. omits NAs.
  mutate(sd_netswing = sd(netswing,na.rm=TRUE)) %>% 
  mutate(plpr = 1- pt((seat_gap - mean(netswing,na.rm = TRUE))/sd_netswing,df=n_netswing-1)) #plurality loss probability; sd(netswing) uses ALL elections, pre & post.



#######  Merging parlgov (aka, pg) and deu #######

deu <- read.csv("deu_positions_master.csv")
deu$plpr <- NULL #is empty in master and don't want two in merged df

# date format for merging var in pg
pg$elec_date_iso <- as.Date(as.character(pg$election_date), format = "%Y-%m-%d")

#create object with pg variables to be merged
pg2 <- data.frame(pg$elec_date_iso, pg$plpr, pg$netswing, pg$sd_netswing, pg$country_name_short, pg$seat_gap, pg$plurlost, pg$second2plur)
pg2$elec_date_iso <- pg2$pg.elec_date_iso
pg2$pg.elec_date_iso <-  NULL
pg2$plpr <- pg2$pg.plpr
pg2$pg.plpr <- NULL
pg2$netswing <- pg2$pg.netswing
pg2$pg.netswing <- NULL
pg2$country_name_short <- pg2$pg.country_name_short
pg2$pg.country_name_short <- NULL
pg2$sd_netswing <- pg2$pg.sd_netswing
pg2$pg.sd_netswing <- NULL
pg2$seat_gap <- pg2$pg.seat_gap
pg2$pg.seat_gap <- NULL
pg2$plurlost <- pg2$pg.plurlost
pg2$pg.plurlost <- NULL
pg2$second2plur <- pg2$pg.second2plur
pg2$pg.second2plur <- NULL

# date format for merging var in deu
deu$elec_date_iso <- as.Date(as.character(deu$date_elec), format = "%m/%d/%Y")
#str(deu$elec_date_iso)

# create country_name_short in pg for merge
pg2$Country <- recode_factor(factor(pg$country_name_short),
                            AUS="au", AUT="at", BEL="be", BGR="bu", CAN="ca",
                            CHE="ch", CYP="cy", CZE="cz", DEU="de", DNK="dk",
                            ESP="es", EST="ee", FIN="fi", FRA="fr", GBR="uk",
                            GRC="el", HRV="hr", HUN="hu", IRL="ie", ISL="is",
                            ISR="il", ITA="it", JPN="jp", LTU="lt", LUX="lu",
                            LVA="lv", MLT="mt", NLD="nl", NOR="no", NZL="nz",
                            POL="pl", PRT="pt", ROU="ro", SVK="sk", SVN="si",
                            SWE="se", TUR="tr")

#create unique identifier
pg2$cty_elec <- paste(pg2$Country, pg2$elec_date_iso, sep="-") #concatenate
#str(pg2$cty_elec)

deu$cty_elec <- paste(deu$Country, deu$elec_date_iso, sep="-")

#merge
m <- merge(deu,pg2,by="cty_elec",all=TRUE)#old: all.deu=TRUE. 


#drop the non-EU cases
dim(m)
#m <- m %>% drop_na(DEU.Proposal.Number)
m <- m %>% filter(!is.na(DEU.Proposal.Number)) %>% droplevels()
dim(m)

#save PLPR
write.csv(m, "PLPR.csv")




### APPENDIX D ###
########## PLPR Validation ###########
# uses PLPRs with 1990 start date, as in JEPP paper



## TABLE A4 ##
## Predictive validity for plurlost ##
# match JEPP sample
eu <- c('AUT','BEL','BGR','CYP','CZE','DEU','DNK','EST','GRC','ESP','FIN','FRA','HRV','HUN','IRL','ITA'
            ,'LTU','LUX','LVA','MLT','NLD','POL','PRT','ROU','SWE','SVN','SVK','GBR')
pg2$eu <- pg2$country_name_short %in% eu 
pg2$eu <- as.numeric(pg2$eu)
rm(eu)
pgeu <- pg2[which(pg2$eu==1),] # create eu only sample

#logit
library(ggplot2)
library(stargazer)
mylm <- lm(second2plur ~ plpr + Country, data=pgeu) # avg marginal effect =.79
mylogit <- glm(second2plur ~ plpr + Country, data=pgeu, family="binomial") # strata fn implicit
summary(mylogit)
stargazer(mylogit, single.row=TRUE, star.char="", notes="standard errors in parentheses", notes.append=FALSE,type = "latex")




## FIGURE A1 ##
##### Cross-validation with KL2015 LPR measure #####
# create common merge variable in pg that matches LPR DF id var.  eg, AUT1997_8
pgeu$year <- substring(as.character.Date(pgeu$elec_date_iso),1,4)
pgeu$month <- substring(as.character.Date(pgeu$elec_date_iso),6,7)
  pgeu$month <- ifelse(pgeu$month=="01","_1",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="02","_2",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="03","_3",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="04","_4",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="05","_5",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="06","_6",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="07","_7",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="08","_8",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="09","_9",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="10","_10",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="11","_11",pgeu$month)
  pgeu$month <- ifelse(pgeu$month=="12","_12",pgeu$month)
pgeu$case <- paste(as.character(pgeu$country_name_short),pgeu$year,pgeu$month,sep="")

#merge
kl <- read.csv("lprdata_distrib_augmented_2015.csv",sep=";")
lpr <- data.frame(kl$case,kl$lpr)
lpr$case <- lpr$kl.case
plpr <- data.frame(pgeu$case,pgeu$plpr)
plpr$case <- plpr$pgeu.case 
lpr.plpr <- merge(lpr,plpr,by="case",all=TRUE)#old: all.deu=TRUE. 

# ggplot w labels
library(ggplot2)
ggplot(lpr.plpr,aes(x=kl.lpr, y=pgeu.plpr))+
  geom_point() +
  geom_text(label=lpr.plpr$case) +
  geom_abline(intercept = 0, slope = 1)+
  xlab("Plurality Loss Probabilities (KL2015)") +
  ylab("Plurality Loss Probabilities (New)")



## FIGURE A2 ##
hist(lpr.plpr$pgeu.plpr,breaks=28,xlab="",main="New Plurality Loss Probabilities")
hist(lpr.plpr$kl.lpr,breaks=28, xlab="", main="KL2015 Plurality Loss Probabilities")
summary(lpr.plpr$pgeu.plpr)
str(lpr.plpr$pgeu.plpr)
summary(lpr.plpr$kl.lpr )
str(lpr.plpr$kl.lpr)



## FIGURE A3 ##
### Face validity: Boxplots
# since 1990
m$bymedian <- with(m,reorder(Country.y,plpr, median))
boxplot(plpr ~ bymedian, data=m, col="lightblue", ylab="Plurality Loss Probability")

